import React, { ReactNode } from "react"
import useSWR from "swr"
import { createContext, useContext } from 'react';

import { decAnyMetadata, unifyMetadata, type UnifiedMetadata } from "@polkadot-api/substrate-bindings"
import { useUnstableProvider } from "./useUnstableProvider"

const MetadataProvider = createContext<UnifiedMetadata | undefined>(undefined)
export const useMetadata = () => useContext(MetadataProvider)

interface MetadataProviderProps {
    children: ReactNode
}

export const MetadataProviderProvider: React.FC<MetadataProviderProps> = ({ children }) => {
    const { client, chainId } = useUnstableProvider()
    const { data: metadata } = useSWR(
        client && chainId ? ["metadata", client, chainId] : null,
        async ([_, client]) => {
            const storageKey = `metadata-${chainId}`
            const storedMetadata = sessionStorage.getItem(storageKey)
            if (storedMetadata) return unifyMetadata(decAnyMetadata(storedMetadata))

            const metadata = await new Promise<string | undefined>((resolve, reject) =>
                    client._request("state_getMetadata", [], {
                        onSuccess: resolve,
                        onError: reject,
                    }),
                )
                .then(r => r)
                .catch(_ => undefined)

            if (metadata) {
                sessionStorage.setItem(storageKey, metadata)
                return unifyMetadata(decAnyMetadata(metadata))
            } else {
                sessionStorage.removeItem(storageKey)
                return undefined
            }
        }
    )

    return (
        <MetadataProvider.Provider value={metadata}>
            {children}
        </MetadataProvider.Provider>
    )
}
