import useSWRSubscription from "swr/subscription"
import { switchMap, from } from "rxjs"
import { useUnstableProvider } from "./useUnstableProvider"

type SystemHealth = {
    isSyncing: Boolean
    shouldHavePeers: Boolean
    peers: number
}

export const useSystemHealth = () => {
    const { chainId, client, chainHead$ } = useUnstableProvider()
    const { data: systemHealth } = useSWRSubscription(
        chainId && client && chainHead$ ? ["system_health", chainId, client, chainHead$] : null,
        ([method, chainId, client, chainHead$], { next }) => {
            const { best$ } = chainHead$
            const subscription = best$?.pipe(
                    switchMap(() => from(new Promise((resolve, reject) =>
                        client._request("system_health", [], {
                            onSuccess: resolve,
                            onError: reject,
                        })
                    )))
                )
                .subscribe({
                    next(data: SystemHealth) {
                        next(null, data)
                    },
                    error: next
                })
            return () => subscription?.unsubscribe()
        }
    )
    return systemHealth
}
